#define MAX_TITLE_LEN 69
#define LAB_LEN 12
#define MAX_CMND_LEN 100

#include "menu.h"

typedef struct {
	char	nd_label[LAB_LEN+1];		/* short label, e.g.  PostOf */
	int	nd_type;			/* type, e.g. TYPE_COMMAND */
	char	nd_title[MAX_TITLE_LEN+1];	/* title, e.g. "Post Office" */

	/* following filled in only for menus (nd_n set to 0 for commands)*/
	int	nd_n;				/* number of entries in menu */
	char	nd_labs[MAX_MENU_ENTRIES][LAB_LEN+1]; /* labels */
	int	nd_nums[MAX_MENU_ENTRIES];	/* node numbers */
	MENU	*nd_menu;			/* printable menu */

	/* following for commands */
	char	nd_command[MAX_CMND_LEN];	/* text of command */
} NODE;

#define MAX_NODES 120
extern int n_nodes;
extern NODE *node[];

#define TYPE_MENU 1
#define TYPE_COMMAND 2
