#define ACTION_NONE		0
#define ACTION_ENTER		1
#define ACTION_EXIT		2
#define ACTION_MAIN		3
#define ACTION_QUIT		4
#define ACTION_REDRAW		5
#define ACTION_HELP		6
#define ACTION_VIEW_SHORT_CUTS	7
#define ACTION_ADD_ITEM		8
#define ACTION_ADD_MENU		9
#define ACTION_DELETE_ITEM	10
#define ACTION_SHORT_CUT	11
#define ACTION_LINE_HELP	12
