extern char user_logon[];
extern char user_home[];
extern char base_dir[];
extern char menu_letter[];
extern int  cur_level;
extern int  sc_map[];
extern int  short_cut_index;
extern int  show_intro;
extern int  cur_line_no;

#define version "2.01"
#define MENU_DIR "/usr/local/uclink/menu2.01"
#define HEADER "-- UCLink Menu System, v2.01 --"
