
/*
 * Copyright 1993, 1994, 1995 by the Regents of the University of California
 * see the file "Copyright" in the distribution for conditions of use.
 */

#include "misc.h"

/* the first time, show the introductory messages */

intro_message()
{
	char command[100];

	strcpy( command, "more -d " );
	strcat( command, base_dir );
	strcat( command, "/etc/Intro.text" );
	system( command );
}
